@@ -115,11 +115,30 @@ void test_String()
     test_assert( "string convert to std::string", w, w2 );
 
     Py::String r2( "12345 789" );
-    Py::Char c6 = r2[5];
-    test_assert( "string convert to std::string", c6, blank );
 
-    Py::Char c7 = r2.front();
-    Py::Char c8 = r2.back();
+    // ord tests
+    Py::Char c3 = r2[2];
+    long v3 = c3.ord();
+    test_assert( "string ord value < 2^7", v3, long( 0x33 ) );
+
+    Py::Char c4( 0xd5 );
+    long v4 = c4.ord();
+    test_assert( "string ord value < 2^8", v4, long( 0xd5 ) );
+
+    Py::Char c5( 0x10437 );
+    long v5 = c5.ord();
+    test_assert( "string ord value > 2^16", v5, long( 0x10437 ) );
+
+    Py::Char c6( 0x10ff00 );
+    long v6 = c6.ord();
+    test_assert( "string ord value > 2^16", v6, long( 0x10ff00 ) );
+
+    // convert tests
+    Py::Char c7 = r2[5];
+    test_assert( "string convert to std::string", c7, blank );
+
+    Py::Char c8 = r2.front();
+    Py::Char c9 = r2.back();
 }
 
 void test_boolean()
@@ -221,7 +240,6 @@ void test_long()
     py_long1 = cxx_long1;
     py_long2 = cxx_long2;
     py_long3 = cxx_long3;
-   
 
     // ------------------------------------------------------------
     test_assert( "long operator ==", cxx_long2 == cxx_long3, py_long2  == py_long3 );
@@ -288,6 +306,21 @@ void test_long()
     // ------------------------------------------------------------
     test_assert( "long operator long", cxx_long2, long( py_long2 ) );
     test_assert( "long operator int", int( cxx_long2 ), int( py_long2 ) );
+
+#ifdef HAVE_LONG_LONG
+    long long cxx_long_long11 = 1152921504616856976ll;
+    long long cxx_long_long12 = 6152921504616856976ll;
+    Py::Long py_long11( cxx_long_long11 );
+    Py::Long py_long12( 100 );
+
+    test_assert( "long long constructor 1", cxx_long_long11, py_long11.as_long_long() );
+    test_assert( "long long constructor 2", 1152921504616856976ll, py_long11.as_long_long() );
+    test_assert( "long long constructor 4", 100ll, py_long12.as_long_long() );
+
+    py_long12 = cxx_long_long12;
+
+    test_assert( "long long operator=", cxx_long_long12, py_long12.as_long_long() );
+#endif
 }
 
 void test_float()
@@ -558,7 +591,7 @@ void test_Dict()
     test_assert( "dict iterator not end != [2]", true, it1 != dict1.end() );
     test_assert( "dict iterator not end == [2]", false, it1 == dict1.end() );
     ++it1;
-    
+
     Py::Dict::iterator it2 = dict1.end();
     bool x = it1 != it2;
     test_assert( "x", false, x );
